/* Copyright (C) 2002 MySQL AB & Jorge del Conde

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
    
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA 
*/

#ifndef CTABLEWINDOW_H
#define CTABLEWINDOW_H

#include <qvariant.h>
#include "CMyWindow.h"
#include "CMySQLConnection.h"
#include "CMySQLQuery.h"
#include "CMessageWindow.h"
#include "Config.h"
#include "CDatabaseTreeItem.h"
#include <qintdict.h>
#include <qtable.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qvalidator.h>

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QAction;
class QActionGroup;
class QToolBar;
class QPopupMenu;
class QTabWidget;
class QButtonGroup;
class QRadioButton;
class QTable;
class QWidget;
class QLabel;
class QWidget;
class QComboBox;

class p_FieldInfo
{
public:
  p_FieldInfo(int i)
  {    
    id = i;
    Type = QString::null;
    Default = QString::null;
    Value = QString::null;
    FieldName = QString::null;
    Length = QString::null;
    Unsigned = false;
    Binary = false;
    ZeroFill = false;
    AutoIncrement = false;
    Null = false;
    originalFieldName = QString::null;
    newField = false;
  }
  
  p_FieldInfo(const p_FieldInfo &f)
  {
    id = -1;
    Type = f.Type;
    Default = f.Default;
    Value = f.Value;
    FieldName = f.FieldName;
    Length = f.Length;
    Unsigned = f.Unsigned;
    Binary = f.Binary;
    ZeroFill = f.ZeroFill;
    AutoIncrement = f.AutoIncrement;
    Null = f.Null;
    originalFieldName = QString::null;
    newField = false;
  }  

  int getID() { return id; }
  void setID(int i) { id = i; }  
  QString Type;
  QString Default;
  QString Value;
  QString Length;
  QString FieldName;
  bool Unsigned;
  bool Binary;
  bool ZeroFill;
  bool AutoIncrement;
  bool Null;

  bool newField;
  QString originalFieldName;

  bool operator== (p_FieldInfo f) const
  {
    bool ret = true;
    if (&f != this)
    {
      ret = (f.AutoIncrement == AutoIncrement);
      ret &= (f.Type == Type);
      ret &= (f.Default == Default);
      ret &= (f.Value == Value);
      ret &= (f.Length == Length);
      ret &= (f.FieldName == FieldName);
      ret &= (f.Unsigned == Unsigned);
      ret &= (f.Binary == Binary);
      ret &= (f.ZeroFill == ZeroFill);
      ret &= (f.Null == Null);
    }
    return ret;
  }

private:
  int id;
};

class CComboFieldItem : public QComboTableItem
{
public:
  CComboFieldItem(QTable * table, int fieldid, const QStringList & list, bool editable = false);
  CComboFieldItem(QTable * table, p_FieldInfo *f, const QStringList & list, bool editable = false);
  CComboFieldItem(QTable * table, p_FieldInfo *f, int fieldid, const QStringList & list, bool editable = false);
  ~CComboFieldItem();
  p_FieldInfo *Field;
};


class CAlterTableOptions : public QWidget
{ 
  Q_OBJECT
    
public:
  CAlterTableOptions(QWidget* parent, QTable *f, CMySQLConnection *m, const char* name = 0, WFlags fl = 0);
  ~CAlterTableOptions();
  QString getAlterTableOptions();
  QComboBox* orderBy;
  QCheckBox* ignore;
  
public slots:
  void reset();
  void refresh();
  
private:
  QTable *Fields;
  CMySQLConnection *mysql;
  QLabel* TextLabel1;
  QGridLayout* CAlterTableOptionsLayout;
};

class CTblProperties : public QWidget
{ 
  Q_OBJECT
    
public:
  CTblProperties(QWidget* parent, CMySQLConnection *m, const char* name = 0, WFlags fl = 0);  
  ~CTblProperties();
  QString getTableProperties();
  QLineEdit* tableName;
  QLineEdit* comment;
  QLineEdit* minRows;
  QLineEdit* maxRows;
  QLineEdit* autoIncrement;
  QComboBox* tableType;
  QComboBox* rowFormat;
  QComboBox* packKeys;
  QComboBox* delayKeyWrite;
  QComboBox* checksum;
  QLineEdit* avgRowLength;

private:
  QRegExpValidator *longValidator;
  CMySQLConnection *mysql;
  QLabel* TextLabel1;
  QLabel* TextLabel5;
  QLabel* TextLabel10;
  QLabel* TextLabel11;
  QLabel* TextLabel8;
  QLabel* TextLabel9;
  QLabel* TextLabel2;
  QLabel* TextLabel3;
  QLabel* TextLabel4;
  QLabel* TextLabel6;
  QLabel* TextLabel7;
  QGridLayout* CTblPropertiesLayout;
};

class CTableIndexes : public QWidget
{ 
  Q_OBJECT
    
public:
  enum IndexType { PRIMARY, INDEX, UNIQUE, FULLTEXT };
  CTableIndexes(QTable *table, CMessagePanel *m, QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
  ~CTableIndexes();
  QString getIndexes(bool alter=false);
  QString getDropIndexes();
  void setMySQL(CMySQLConnection *m) { mysql = m; }
  void setIsAlterTable(bool b) { isAlterTable = b; }
  QComboBox* indexName;

public slots:
  void refresh();
  void singleRemove(const QString &keyname, const QString &field);
  void singleAdd(const QString &keyname, const QString &field);
  void Add(const QString &keyname, IndexType t, QStringList fields);  
  void ChangeType(bool b, IndexType t);
  void clear();

signals:
  void primaryKey(bool, int);
  
private slots:
  void RemoveField();
  void AddField();
  void AddIndex();
  void DeleteIndex();  
  void IndexChanged(const QString &s);
  void UniToggled(bool);
  void IdxToggled(bool);
  void FtToggled(bool);
  void UniFtToggled(bool);

private:
  class Index
  {
  public:
    Index(const QString &name, IndexType t = INDEX)      
    {
      Name = name;
      Type = t;
      UniFullIndex = false;
      isNewIndex = false;
    }
    IndexType Type;
    bool UniFullIndex;
    bool isNewIndex;
    QString Name;
    QStringList indexSort;
    QDict<p_FieldInfo> indexFields;    
  };
  bool isAlterTable;
  QString getIndexFields(Index *index);
  QTable *Fields;
  QGridLayout* CTableIndexesLayout;
  QGridLayout* indexTypesLayout;
  QLabel* TextLabel3;
  QLabel* TextLabel4;
  QListBox* fieldsUsed;
  QListBox* availableFields;
  QLabel* TextLabel1;  
  QButtonGroup* indexTypes;
  QRadioButton* uniqueIndex;
  QRadioButton* fullTextIndex;
  QRadioButton* indexIndex;
  QCheckBox* uniFullIndex;
  QLabel* PixmapLabel2;
  QLabel* PixmapLabel3;
  QLabel* PixmapLabel4;  
  QPushButton* removeField;
  QPushButton* addField;
  QPushButton* deleteIndex;
  QPushButton* addIndex;
  QDict<Index> Indexes;
  QStringList deletedIndexes;
  CMessagePanel *messagePanel;
  CMySQLConnection *mysql;
  void init();
};

class CFieldProperties : public QWidget
{ 
  Q_OBJECT
    
public:
  CFieldProperties(QWidget* parent, CMessagePanel *m, const char* name = 0, WFlags fl = 0);
  ~CFieldProperties();    
  void setField(p_FieldInfo *field);
  void setFieldType(const QString &t);

private slots:
  void setDefault(const QString &s);
  void setValue(const QString &s);
  void setLength(const QString &s);
  void setUnsigned(bool b);
  void setBinary(bool b);
  void setZeroFill(bool b);
  void setAutoIncrement(bool b);

private:
  class p_LineEdit : public QLineEdit
  {
  public:
    p_LineEdit(QWidget * parent, const char * name = 0)
      : QLineEdit(parent, name)
    {
    }
  public slots:
    void setEnabled(bool b)
    {
    /*  if (!b)
        setText(QString::null);*/
      QLineEdit::setEnabled(b);
    }
  };

  class p_CheckBox : public QCheckBox
  {
  public:
    p_CheckBox (QWidget * parent, const char * name = 0)
      : QCheckBox(parent, name)
    {
    }

  public slots:
    void setEnabled(bool b)
    {
      /*if (!b)
        setChecked(false);*/
      QWidget::setEnabled(b);
    }
  };

  p_FieldInfo *Field;
  p_LineEdit* Default;
  p_LineEdit* Value;
  p_LineEdit* Length;
  p_CheckBox* Unsigned;
  p_CheckBox* Binary;
  p_CheckBox* ZeroFill;
  p_CheckBox* AutoIncrement;
  QIntValidator *intValidator;
  QRegExpValidator *floatValidator;
  CMessagePanel *messagePanel;
  p_FieldInfo *AutoIncField;
  bool skip;

  QLabel* TextLabel1;
  QLabel* TextLabel2;
  QLabel* TextLabel3;
  QGridLayout* CFieldPropertiesLayout;
};

class CTableWindow : public CMyWindow
{ 
  Q_OBJECT
    
public:
  CTableWindow(QWidget* parent, CMySQLConnection *m, CDatabaseTreeItem *item, const QString &tablename);
  CTableWindow(QWidget* parent, CMySQLConnection *m, const QString &tablename, QObject *o, const char *slot);  
  ~CTableWindow();
  QStringList *FieldTypes;
  QStringList *DeletedFields;  //Stores deleted fields when isAlterTable == true  
  bool checkDuplicateNames(const QString &txt, int row, int col);

public slots:
  void addPrimaryIcon(bool add, int row);

protected:
  virtual void setWindowCaption();

signals:
  void doRefresh();
  void valueChanged();
  
private slots:
  void refreshCaller();
  void fieldsValueChanged(int row, int col);
  void fieldsClicked(int row, int, int button, const QPoint &);
  void showContextMenu(int row, int col, const QPoint & pos);  
  void insertRow();
  void deleteRow();
  void reset();
  void copy();
  void paste();
  bool save();
  void saveAs();
  void ShowMessages();
  bool createTable(const QString &tblName);
  bool alterTable();
  bool exec(const QString &qry);
  void setPk();
  void setPrimaryKey(int row, bool b);
  
private:
  void swapSections(int oldIdx, int newIdx);
  QIconSet pkIconSet;
  QPixmap pkIcon;
  QPixmap emptyPkIcon;
  QString getFieldSpecs(p_FieldInfo *field, QStringList &dups, bool &ok);
  QGridLayout* CTableWindowLayout;
  QTable* Fields;
  QTabWidget* Tab;
  CFieldProperties* fieldPropertiesTab;
  CTableIndexes* indexesTab;
  CTblProperties* tablePropertiesTab;
  CAlterTableOptions* alterTableOptionsTab;
  QMenuBar *menubar;
  QPopupMenu *fileMenu;
  QPopupMenu *editMenu;
  QPopupMenu *tableMenu;
  QToolBar *fileBar;
  QToolBar *tableBar;
  QAction* fileSaveAction;
  QAction* fileSaveAsAction;
  QAction* fileCloseAction;
  QAction* fileReloadAction;
  QAction* tableInsertAction;
  QAction* tableDeleteAction;
  QAction* showMessagesAction;
  QAction* editCopyAction;
  QAction* editPasteAction;
  QAction* tablePrimaryKeyAction;
  CMySQLConnection *mysql;
  CDatabaseTreeItem *tableItem;
  bool isAlterTable;
  bool hasPaste;
  bool hasModified;
  p_FieldInfo *pasteField;
  QString tableName;
  int defaultFieldType;
  QString defaultFieldLength;  //for char & varchar
  void init(bool ok);
  void refreshInternalIdx();
  void insert(p_FieldInfo *f = 0);
  void createForm(CMySQLConnection *m, const QString &tablename);
  void closeEvent(QCloseEvent * e);
};

#endif // CTABLEWINDOW_H
