/* Copyright (C) 2002 MySQL AB & Jorge del Conde

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
    
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA 
*/

#include <qapplication.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>
#include <qmessagebox.h>
#include "CHistoryView.h"
#include "Globals.h"

CHistoryView::CHistoryView()
{
}

CHistoryView::~CHistoryView()
{
  write();
}

QString CHistoryView::getHistoryFile()
{
  return QDir::homeDirPath() + "/." + QString(CODENAME) + "/" + QString(HISTORY_FILE_NAME);
}

void CHistoryView::load()
{
  QFileInfo fi(getHistoryFile());
  uint size = fi.size();
  if (size > 0)
  {    
    QFile f_strm(getHistoryFile());
    if (f_strm.open(IO_ReadOnly) && size > 0)
    {
      if (!historyList.empty())
        historyList.clear();
      char *buff = new char [size];
      f_strm.readBlock (buff, size);
      f_strm.close();
      buff[size - 1] = 0;
      QString buffer(buff);
      delete [] buff;
      if (!buffer.isEmpty())
      {
        bool quote = false;
        QString line = QString::null;
        for (uint i = 0; i < buffer.length(); i++)
        {
          QChar c = buffer.at(i);
          if (!quote && (c == '\"'))
            quote = true;
          else
            if (quote)
            {
              if ((c == '\\') && (buffer.at(i+1) == '\"'))
              {
                line += '\"';
                i++;
              }
              else
                if (c == '\"')
                {
                  quote = false;
                  historyList.append(line);
                  line = QString::null;
                }
                else
                  line += c;
            }            
        }
        truncate();
      }
    }
  }
}

void CHistoryView::loadTo(CHistoryPanel *h)
{
  if (historyList.isEmpty())
    return;  
  truncate();  
  for ( QStringList::Iterator it = historyList.begin(); it != historyList.end(); ++it )
    h->History(*it, false);
  h->setBottomItem();
}

void CHistoryView::truncate()
{
  if (historyList.empty())
    return;
  int start = historyList.count() - g_historySize;
  if (start > 0)  
    for (int i = 0; i < start; i++)
      historyList.remove(historyList.at(0));
}

void CHistoryView::write()
{
  truncate();
  if (historyList.empty())
    return;
  QFile file(getHistoryFile());
  if (!file.open( IO_WriteOnly ))
  {    
    QMessageBox::critical(0, qApp->translate("CHistoryView", "Error"), qApp->translate("CHistoryView", "An error occurred while saving the history list."));
    return;
  }    
  QTextStream ts( &file );
  for ( QStringList::Iterator it = historyList.begin(); it != historyList.end(); ++it )
  {
    if (!(*it).stripWhiteSpace().isEmpty())    
      ts << "\"" << charReplace(*it, '"', "\\\"") << "\"\n";    
  }    
  file.close();
}

bool CHistoryView::append(const QString &s)
{
  bool ret = false;
  if (!s.stripWhiteSpace().isEmpty())
  {
    if (historyList.empty())
      ret = true;      
    else
       if (*(historyList.end()) != s)  //Apend only if the last item is not the same as "s"
         ret = true;
    if (ret)
      historyList.append(s);
  }  
  return ret;
}
