/* Copyright (C) 2002 MySQL AB & Jorge del Conde

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
    
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA 
*/

#include "CTableWindow.h"
#include "CTableItem.h"
#include "CTableGroupItem.h"

#include <qvariant.h>
#include <qtable.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qwidget.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qinputdialog.h>
#include <qmessagebox.h>

CAlterTableOptions::CAlterTableOptions(QWidget* parent, QTable *f, CMySQLConnection *m, const char* name, WFlags fl)
: QWidget(parent, name, fl)
{
  if (!name)
    setName("CAlterTableOptions");
  Fields = f;
  mysql = m;  
  setCaption(trUtf8("Alter Table Options"));
  CAlterTableOptionsLayout = new QGridLayout(this, 1, 1, 4, 2, "CAlterTableOptionsLayout"); 
  QSpacerItem* spacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
  CAlterTableOptionsLayout->addItem(spacer, 2, 2);
  
  TextLabel1 = new QLabel(this, "TextLabel1");
  TextLabel1->setText(trUtf8("Order By"));
  
  CAlterTableOptionsLayout->addWidget(TextLabel1, 1, 0);
  
  ignore = new QCheckBox(this, "ignore");
  ignore->setText(trUtf8("IGNORE"));
  
  CAlterTableOptionsLayout->addMultiCellWidget(ignore, 0, 0, 0, 1);
  
  orderBy = new QComboBox(false, this, "orderBy");
  
  CAlterTableOptionsLayout->addWidget(orderBy, 1, 1);
  QSpacerItem* spacer_2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
  CAlterTableOptionsLayout->addItem(spacer_2, 0, 3);
}

CAlterTableOptions::~CAlterTableOptions()
{
}

QString CAlterTableOptions::getAlterTableOptions()
{
  QString ret = QString::null;
  if (orderBy->currentText() != "[DEFAULT]")
    ret = ", ORDER BY " + mysql->Quote(orderBy->currentText());
  return ret;
}

void CAlterTableOptions::reset()
{  
  orderBy->clear();
  orderBy->insertItem("[DEFAULT]");
  orderBy->setCurrentItem(0);
  ignore->setChecked(false);
}

void CAlterTableOptions::refresh()
{
  QString current = orderBy->currentText();
  orderBy->clear();
  orderBy->insertItem("[DEFAULT]");
  for (int i = 0; i < Fields->numRows(); i++)
  {
    QString opt = Fields->text(i, 0);
    if (!opt.isEmpty())
      orderBy->insertItem(Fields->text(i, 0));
  }
  orderBy->setCurrentText(current);
}

CTblProperties::CTblProperties(QWidget* parent, CMySQLConnection *m, const char* name, WFlags fl)
: QWidget(parent, name, fl)
{
  if (!name)
    setName("CTblProperties");
  mysql = m;
  setCaption(trUtf8("Table Properties"));
  longValidator = new QRegExpValidator (QRegExp("\\d{0,10}"), this);
  setCaption(trUtf8("Table Properties"));
  CTblPropertiesLayout = new QGridLayout(this, 1, 1, 4, 2, "CTblPropertiesLayout"); 
  
  TextLabel1 = new QLabel(this, "TextLabel1");
  TextLabel1->setText(trUtf8("Table Name"));
  
  CTblPropertiesLayout->addWidget(TextLabel1, 0, 0);
  
  TextLabel5 = new QLabel(this, "TextLabel5");
  TextLabel5->setText(trUtf8("Avg. Row Length"));
  
  CTblPropertiesLayout->addMultiCellWidget(TextLabel5, 4, 5, 0, 0);
  
  TextLabel10 = new QLabel(this, "TextLabel10");
  TextLabel10->setText(trUtf8("Comment"));
  
  CTblPropertiesLayout->addWidget(TextLabel10, 1, 0);
  
  TextLabel11 = new QLabel(this, "TextLabel11");
  TextLabel11->setText(trUtf8("Auto Increment"));
  
  CTblPropertiesLayout->addWidget(TextLabel11, 6, 0);
  QSpacerItem* spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
  CTblPropertiesLayout->addMultiCell(spacer, 4, 5, 2, 2);
  QSpacerItem* spacer_2 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
  CTblPropertiesLayout->addItem(spacer_2, 7, 1);
  
  TextLabel8 = new QLabel(this, "TextLabel8");
  TextLabel8->setText(trUtf8("Delay Key Write"));
  TextLabel8->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  
  CTblPropertiesLayout->addMultiCellWidget(TextLabel8, 3, 3, 4, 5);
  
  TextLabel9 = new QLabel(this, "TextLabel9");
  TextLabel9->setText(trUtf8("Row Format"));
  TextLabel9->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  
  CTblPropertiesLayout->addWidget(TextLabel9, 1, 5);
  
  TextLabel2 = new QLabel(this, "TextLabel2");
  TextLabel2->setText(trUtf8("Table Type"));
  TextLabel2->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  
  CTblPropertiesLayout->addWidget(TextLabel2, 0, 5);
  
  TextLabel3 = new QLabel(this, "TextLabel3");
  TextLabel3->setText(trUtf8("Min Rows"));
  
  CTblPropertiesLayout->addWidget(TextLabel3, 2, 0);
  
  TextLabel4 = new QLabel(this, "TextLabel4");
  TextLabel4->setText(trUtf8("Max Rows"));
  
  CTblPropertiesLayout->addWidget(TextLabel4, 3, 0);
  
  TextLabel6 = new QLabel(this, "TextLabel6");
  TextLabel6->setText(trUtf8("Pack Keys"));
  TextLabel6->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  
  CTblPropertiesLayout->addMultiCellWidget(TextLabel6, 2, 2, 4, 5);
  
  TextLabel7 = new QLabel(this, "TextLabel7");
  TextLabel7->setText(trUtf8("Checksum"));
  TextLabel7->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  
  CTblPropertiesLayout->addMultiCellWidget(TextLabel7, 4, 4, 3, 5);
  
  tableName = new QLineEdit(this, "tableName");
  
  CTblPropertiesLayout->addMultiCellWidget(tableName, 0, 0, 1, 4);
  
  comment = new QLineEdit(this, "comment");
  
  CTblPropertiesLayout->addMultiCellWidget(comment, 1, 1, 1, 4);
  
  minRows = new QLineEdit(this, "minRows");
  minRows->setValidator(longValidator);
  
  CTblPropertiesLayout->addWidget(minRows, 2, 1);
  
  maxRows = new QLineEdit(this, "maxRows");
  maxRows->setValidator(longValidator);
  
  CTblPropertiesLayout->addWidget(maxRows, 3, 1);
  
  autoIncrement = new QLineEdit(this, "autoIncrement");
  autoIncrement->setValidator(longValidator);
  
  CTblPropertiesLayout->addMultiCellWidget(autoIncrement, 5, 6, 1, 1);
  
  tableType = new QComboBox(false, this, "tableType");
  tableType->insertItem("MyISAM");
  tableType->insertItem("InnoDB");
  tableType->insertItem("HEAP");
  tableType->insertItem("BDB");
  tableType->insertItem("ISAM");
  
  CTblPropertiesLayout->addWidget(tableType, 0, 6);
  
  rowFormat = new QComboBox(false, this, "rowFormat");
  rowFormat->insertItem("DEFAULT");
  rowFormat->insertItem("DYNAMIC");
  rowFormat->insertItem("FIXED");
  rowFormat->insertItem("COMPRESSED");
  rowFormat->setMinimumSize(QSize(100, 0));
  
  CTblPropertiesLayout->addWidget(rowFormat, 1, 6);
  
  packKeys = new QComboBox(false, this, "packKeys");
  packKeys->insertItem("DEFAULT");
  packKeys->insertItem("0");
  packKeys->insertItem("1");
  
  CTblPropertiesLayout->addWidget(packKeys, 2, 6);
  
  delayKeyWrite = new QComboBox(false, this, "delayKeyWrite");
  delayKeyWrite->insertItem("[DEFAULT]");
  delayKeyWrite->insertItem("0");
  delayKeyWrite->insertItem("1");
  
  CTblPropertiesLayout->addWidget(delayKeyWrite, 3, 6);
  
  checksum = new QComboBox(false, this, "checksum");
  checksum->insertItem("[DEFAULT]");
  checksum->insertItem("0");
  checksum->insertItem("1");
  checksum->setMinimumSize(QSize(100, 0));
  
  CTblPropertiesLayout->addWidget(checksum, 4, 6);
  
  avgRowLength = new QLineEdit(this, "avgRowLength");
  avgRowLength->setValidator(longValidator);
  
  CTblPropertiesLayout->addWidget(avgRowLength, 4, 1);

  setTabOrder(tableName, tableType);
  setTabOrder(tableType, comment);
  setTabOrder(comment, rowFormat);
  setTabOrder(rowFormat, minRows);
  setTabOrder(minRows, packKeys);
  setTabOrder(packKeys, maxRows);
  setTabOrder(maxRows, delayKeyWrite);
  setTabOrder(delayKeyWrite, avgRowLength);
  setTabOrder(avgRowLength, checksum);
  setTabOrder(checksum, autoIncrement);
}

CTblProperties::~CTblProperties()
{
  delete longValidator;
}

QString CTblProperties::getTableProperties()
{
  int ver = mysql->getMySQLServerVersion().major;
  QString ret = "TYPE=" + tableType->currentText().upper() + " ";

  if (!minRows->text().stripWhiteSpace().isEmpty())
    ret += "MIN_ROWS=" + minRows->text().stripWhiteSpace() + " ";

  if (!maxRows->text().stripWhiteSpace().isEmpty())
    ret += "MAX_ROWS=" + maxRows->text().stripWhiteSpace() + " ";

  if (!autoIncrement->text().stripWhiteSpace().isEmpty())
    ret += "AUTO_INCREMENT=" + autoIncrement->text().stripWhiteSpace() + " ";

  if (!avgRowLength->text().stripWhiteSpace().isEmpty())
    ret += "AVG_ROW_LENGTH=" + avgRowLength->text().stripWhiteSpace() + " ";
    
  if (packKeys->currentText() != "DEFAULT" || ver > 3)
  ret += "PACK_KEYS=" + packKeys->currentText() + " ";

  if (checksum->currentText() != "[DEFAULT]")
    ret += "CHECKSUM=" + checksum->currentText() + " ";

  if (delayKeyWrite->currentText() != "[DEFAULT]")
    ret += "DELAY_KEY_WRITE=" + delayKeyWrite->currentText() + " ";

  ret += "ROW_FORMAT=" + rowFormat->currentText() + " ";

  QString c = comment->text().stripWhiteSpace();
  if (!c.isEmpty())
    ret += "COMMENT='" + mysql->escape(c) + "' ";

  return ret.left(ret.length() - 1);
}

CTableIndexes::CTableIndexes(QTable *table, CMessagePanel *m, QWidget* parent, const char* name, WFlags fl)
: QWidget(parent, name, fl)
{
  Fields = table;
  isAlterTable = false;
  messagePanel = m;
  setCaption(trUtf8("Indexes"));
  if (!name)
    setName("CTableIndexes");
  Indexes.setAutoDelete(true);  
  CTableIndexesLayout = new QGridLayout(this, 1, 1, 4, 2, "CTableIndexesLayout"); 
  QSpacerItem* spacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);
  CTableIndexesLayout->addItem(spacer, 3, 1);
  QSpacerItem* spacer_2 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
  CTableIndexesLayout->addItem(spacer_2, 5, 1);
  QSpacerItem* spacer_3 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Preferred);
  CTableIndexesLayout->addMultiCell(spacer_3, 0, 1, 1, 1);
  
  TextLabel3 = new QLabel(this, "TextLabel3");
  TextLabel3->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 0, 0, TextLabel3->sizePolicy().hasHeightForWidth()));
  TextLabel3->setText(trUtf8("Available Fields"));
  
  CTableIndexesLayout->addWidget(TextLabel3, 0, 0);
  
  TextLabel4 = new QLabel(this, "TextLabel4");
  TextLabel4->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 0, 0, TextLabel4->sizePolicy().hasHeightForWidth()));
  TextLabel4->setText(trUtf8("Fields Used"));
  
  CTableIndexesLayout->addWidget(TextLabel4, 0, 2);
  
  fieldsUsed = new QListBox(this, "fieldsUsed");
  fieldsUsed->setSelectionMode(QListBox::Extended);
  QWhatsThis::add(fieldsUsed, trUtf8("Fields used in this Index"));
  
  CTableIndexesLayout->addMultiCellWidget(fieldsUsed, 1, 5, 2, 2);
  
  availableFields = new QListBox(this, "availableFields");
  availableFields->setSelectionMode(QListBox::Extended);  
  QWhatsThis::add(availableFields, trUtf8("Available Fields that can be used as an Index"));
  
  CTableIndexesLayout->addMultiCellWidget(availableFields, 1, 5, 0, 0);
  
  TextLabel1 = new QLabel(this, "TextLabel1");
  TextLabel1->setText(trUtf8("Index Name"));
  
  CTableIndexesLayout->addWidget(TextLabel1, 0, 3);
  
  deleteIndex = new QPushButton(this, "deleteIndex");
  deleteIndex->setMinimumSize(QSize(20, 20));
  deleteIndex->setMaximumSize(QSize(20, 20));
  deleteIndex->setText(trUtf8(""));
  deleteIndex->setPixmap(getPixmapIcon("minusIcon"));    
  QWhatsThis::add(deleteIndex, trUtf8("Delete selected Index"));
  
  CTableIndexesLayout->addWidget(deleteIndex, 1, 7);
  
  indexName = new QComboBox(false, this, "indexName");  
  indexName->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, indexName->sizePolicy().hasHeightForWidth()));
  QWhatsThis::add(indexName, trUtf8("Available Index Names"));
  
  CTableIndexesLayout->addMultiCellWidget(indexName, 0, 0, 4, 7);
  
  indexTypes = new QButtonGroup(this, "indexTypes");
  indexTypes->setTitle(trUtf8("Index Type"));
  QWhatsThis::add(indexTypes, trUtf8("Type of Index for 'Index Name'"));
  indexTypes->setColumnLayout(0, Qt::Vertical);
  indexTypes->layout()->setSpacing(2);
  indexTypes->layout()->setMargin(4);
  indexTypesLayout = new QGridLayout(indexTypes->layout());
  indexTypesLayout->setAlignment(Qt::AlignTop);
  
  uniqueIndex = new QRadioButton(indexTypes, "uniqueIndex");
  uniqueIndex->setText("UNIQUE");
  
  indexTypesLayout->addWidget(uniqueIndex, 2, 1);
  
  fullTextIndex = new QRadioButton(indexTypes, "fullTextIndex");
  fullTextIndex->setText("FULLTEXT");
  
  indexTypesLayout->addWidget(fullTextIndex, 3, 1);
  QSpacerItem* spacer_4 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
  indexTypesLayout->addItem(spacer_4, 5, 1);
  
  indexIndex = new QRadioButton(indexTypes, "indexIndex");
  indexIndex->setText("INDEX");
  
  indexTypesLayout->addWidget(indexIndex, 1, 1);
  
  PixmapLabel2 = new QLabel(indexTypes, "PixmapLabel2");
  PixmapLabel2->setPixmap(getPixmapIcon("mulIcon"));
  
  indexTypesLayout->addWidget(PixmapLabel2, 1, 0);
  
  PixmapLabel3 = new QLabel(indexTypes, "PixmapLabel3");
  PixmapLabel3->setPixmap(getPixmapIcon("uniIcon"));
  
  indexTypesLayout->addWidget(PixmapLabel3, 2, 0);
  
  PixmapLabel4 = new QLabel(indexTypes, "PixmapLabel4");
  PixmapLabel4->setPixmap(getPixmapIcon("ftIcon"));
  
  indexTypesLayout->addWidget(PixmapLabel4, 3, 0);

  uniFullIndex = new QCheckBox(indexTypes, "UniFullIndex");
  uniFullIndex->setText("INDEX");

  indexTypesLayout->addWidget(uniFullIndex, 4, 1);

  QSpacerItem* spacer_5 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
  indexTypesLayout->addItem(spacer_5, 2, 2);
  
  CTableIndexesLayout->addMultiCellWidget(indexTypes, 2, 5, 3, 7);
  
  removeField = new QPushButton(this, "removeField");
  removeField->setMinimumSize(QSize(30, 30));
  removeField->setMaximumSize(QSize(30, 30));
  removeField->setText("");
  removeField->setPixmap(getPixmapIcon("leftArrowIcon"));    
  
  CTableIndexesLayout->addWidget(removeField, 4, 1);
  
  addField = new QPushButton(this, "addField");
  addField->setMinimumSize(QSize(30, 30));
  addField->setMaximumSize(QSize(30, 30));
  addField->setText("");
  addField->setPixmap(getPixmapIcon("rightArrowIcon"));
    
  CTableIndexesLayout->addWidget(addField, 2, 1);
  QSpacerItem* spacer_6 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
  CTableIndexesLayout->addItem(spacer_6, 1, 4);
  QSpacerItem* spacer_7 = new QSpacerItem(10, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);
  CTableIndexesLayout->addItem(spacer_7, 1, 6);
  
  addIndex = new QPushButton(this, "addIndex");
  addIndex->setMinimumSize(QSize(20, 20));
  addIndex->setMaximumSize(QSize(20, 20));
  addIndex->setText(trUtf8(""));
  addIndex->setPixmap(getPixmapIcon("plusIcon"));  
  QWhatsThis::add(addIndex, trUtf8("Add new Index"));
  
  CTableIndexesLayout->addWidget(addIndex, 1, 5);
  
  setTabOrder(indexName, availableFields);
  setTabOrder(availableFields, fieldsUsed);
  setTabOrder(fieldsUsed, addField);
  setTabOrder(addField, removeField);
  setTabOrder(removeField, addIndex);
  setTabOrder(addIndex, deleteIndex);
  setTabOrder(deleteIndex, indexIndex);
  setTabOrder(indexIndex, uniqueIndex);
  setTabOrder(uniqueIndex, fullTextIndex);
  setTabOrder(fullTextIndex, uniFullIndex);
  clear();
  init();  
}

CTableIndexes::~CTableIndexes()
{    
}

void CTableIndexes::clear()
{
  Indexes.clear();
  deletedIndexes.clear();
  fieldsUsed->clear();
  availableFields->clear();
  indexName->clear();
  indexName->insertItem(getPixmapIcon("pkIcon"), "PRIMARY");
  Indexes.insert("PRIMARY", new Index("PRIMARY", PRIMARY));
  indexName->setCurrentItem(0);
  refresh();  
}

void CTableIndexes::refresh()
{  
  availableFields->clear();
  fieldsUsed->clear();  
  
  QStringList lst;
  lst.clear();
  for (int j = 0; j < Fields->numRows(); j++)
  {
    QString t = Fields->text(j, 0);
    if (!t.isEmpty() && !t.isNull())
      lst.append(t);
  }

  Index *index = Indexes.find(indexName->currentText());

  if (index->Type == PRIMARY)
  {
    uniqueIndex->setEnabled(false);
    fullTextIndex->setEnabled(false);
    indexIndex->setEnabled(false);
    uniFullIndex->setEnabled(false);
    uniFullIndex->setChecked(false);
    uniqueIndex->setChecked(false);
    fullTextIndex->setChecked(false);
    indexIndex->setChecked(false);
    index->UniFullIndex = false;
  }
  else
  {
    uniqueIndex->setEnabled(true);
    fullTextIndex->setEnabled(true);
    indexIndex->setEnabled(true);
    switch (index->Type)
    {
    case UNIQUE:
      uniFullIndex->setEnabled(true);
      uniFullIndex->setChecked(index->UniFullIndex);
      uniqueIndex->setChecked(true);
      break;
    case FULLTEXT:
      uniFullIndex->setEnabled(true);
      uniFullIndex->setChecked(index->UniFullIndex);
      fullTextIndex->setChecked(true);
      break;
    default:  //index->Type can't be PRIMARY ... needs to be INDEX
      index->UniFullIndex = false;
      uniFullIndex->setEnabled(false);
      uniFullIndex->setChecked(false);
      indexIndex->setChecked(true);
      break;
    }
  }

  QStringList del;
  for (QStringList::Iterator it = index->indexSort.begin(); it != index->indexSort.end(); ++it)
  {    
    if (lst.find(*it) == lst.end())  //The field was either renamed or removed
      del.append(*it);
    else
      fieldsUsed->insertItem(*it);
  }
 
  if (!del.empty())  //Remove renamed or deleted indexes from indexFields
    for (QStringList::Iterator d = del.begin(); d != del.end(); ++d)
    {
      index->indexSort.remove(*d);
      index->indexFields.remove(*d);
      index->indexFields.resize(index->indexFields.count());
    }
  
  for (QStringList::Iterator it2 = lst.begin(); it2 != lst.end(); ++it2)
  {
    if (fieldsUsed->findItem(*it2, Qt::CaseSensitive) == 0)
      availableFields->insertItem(*it2);
  }

  if (indexName->currentText().lower() == "primary")
  {
    for (int j = 0; j < Fields->numRows(); j++)
    {
      QString txt = Fields->text(j, 0).stripWhiteSpace();
      if (!txt.isEmpty())
        emit primaryKey((index->indexSort.find(txt) != index->indexSort.end()), j);
      else
        emit primaryKey(false, j);
    }
  }

  addField->setEnabled(availableFields->count() > 0);
  removeField->setEnabled(fieldsUsed->count() > 0);
  deleteIndex->setEnabled(indexName->currentText().lower() != "primary");
}

void CTableIndexes::singleRemove(const QString &keyname, const QString &field)
{
  if (field.isEmpty() || keyname.isEmpty())
   return;
  Index *index = Indexes.find(keyname);
  index->indexSort.remove(field);
  index->indexFields.remove(field);
  index->indexFields.resize(index->indexFields.count());
  refresh();
}

void CTableIndexes::singleAdd(const QString &keyname, const QString &field)
{
  if (field.isEmpty() || keyname.isEmpty())
   return;
  Index *index = Indexes.find(keyname);
  index->indexFields.resize(index->indexFields.count() + 1); 
  for (int j = 0; j < Fields->numRows(); j++)
  {
    CComboFieldItem *c = (CComboFieldItem *)Fields->item(j, 2);
    if (c->Field->FieldName == field)
    {
      index->indexFields.resize(index->indexFields.count() + 1);
      index->indexSort.append(c->Field->FieldName);
      index->indexFields.insert(c->Field->FieldName, c->Field);
      break;
    }
  }
  refresh();
}


void CTableIndexes::RemoveField()
{
  if (fieldsUsed->count() == 0)
    return;
  Index *index = Indexes.find(indexName->currentText());
  for (uint i = 0; i < fieldsUsed->count(); i++)
  {
    if (fieldsUsed->isSelected(i))
    {
      index->indexSort.remove(fieldsUsed->text(i));
      index->indexFields.remove(fieldsUsed->text(i));      
    }
  }
  index->indexFields.resize(index->indexFields.count());
  refresh();
}

void CTableIndexes::AddField()
{
  if (availableFields->count() == 0)
    return;
  Index *index = Indexes.find(indexName->currentText());
  for (uint i = 0; i < availableFields->count(); i++)
  {
    if (availableFields->isSelected(i))
    {
      for (int j = 0; j < Fields->numRows(); j++)
      {
        CComboFieldItem *c = (CComboFieldItem *)Fields->item(j, 2);
        if (c->Field->FieldName == availableFields->text(i))
        {
          index->indexFields.resize(index->indexFields.count() + 1);
          index->indexSort.append(c->Field->FieldName);
          index->indexFields.insert(c->Field->FieldName, c->Field);
          break;
        }
      }
    }
  }
  refresh();
}

void CTableIndexes::Add(const QString &keyname, IndexType t, QStringList fields)
{
  Index *index = 0;
  if (t != PRIMARY)
  {
    indexName->insertItem(keyname);
    indexName->setCurrentText(keyname);
    Indexes.resize(indexName->count());
    index = new Index(keyname);
    Indexes.insert(keyname, index);
  }
  else
    index = Indexes.find("PRIMARY");

  for (QStringList::Iterator it = fields.begin(); it != fields.end(); ++it)
  {
    for (int j = 0; j < Fields->numRows(); j++)
    {
      CComboFieldItem *c = (CComboFieldItem *)Fields->item(j, 2);
      if (c->Field->FieldName == *it)
      {
        index->indexFields.resize(index->indexFields.count() + 1);
        index->indexSort.append(c->Field->FieldName);
        index->indexFields.insert(c->Field->FieldName, c->Field);
        break;
      }
    }
  }
  indexName->setCurrentText(keyname);
  if (t != PRIMARY)
    ChangeType(true, t);
  else
    refresh();
}

void CTableIndexes::AddIndex()
{
  bool ok = false;
  QString idx = QInputDialog::getText(tr("New Index"), tr("Please enter the Index Name"), QLineEdit::Normal, QString::null, &ok);
  if (ok && !idx.isEmpty())
  {
    if (idx.lower() == "primary")
    {
      messagePanel->Warning(tr("You can not have an Index named 'PRIMARY'"));
      return;
    }
    for (int i = 0; i < indexName->count(); i++)
    {
      if (indexName->text(i).lower() == idx.lower())
      {
        messagePanel->Warning(tr("You already have an index called") + ": " + idx);
        return;
      }
    }
    indexName->insertItem(idx);
    indexName->setCurrentText(idx);
    Indexes.resize(indexName->count());
    Index *index = new Index(idx);
    index->isNewIndex = isAlterTable;    
    Indexes.insert(idx, index);
    ChangeType(true, INDEX);
    refresh();
  }      
}

QString CTableIndexes::getIndexFields(Index *index)
{
  QString ret = QString::null;
  if (!index->indexFields.isEmpty())
  {
    for (QStringList::Iterator it = index->indexSort.begin(); it != index->indexSort.end(); ++it)    
    {
      p_FieldInfo *f = index->indexFields.find(*it);
      QString type = f->Type.lower();
      ret += mysql->Quote(*it);
      if ((type.find("blob") != -1) || (type.find("text") != -1))
      {
        if (f->Length.isEmpty())
        {
          messagePanel->Warning(tr("Field") + " '" + *it + "' " + tr("is part of an Index but no Key Length was specified.  Using 255 as the default Key Length."));
          f->Length = "255";
        }
        ret += "(" + f->Length + ")";
      }
      ret += ", ";
    }
    ret = ret.left(ret.length() - 2);
  }
  return ret;
}

QString CTableIndexes::getDropIndexes()
{
  QString ret = QString::null;
  for (int i = 0; i < indexName->count(); i++)
  {    
    Index *index = Indexes.find(indexName->text(i));
    if (!index->isNewIndex)
    {
      ret += "DROP ";
      switch (index->Type)
      {
      case PRIMARY: ret += "PRIMARY KEY, ";
        break;
      default: ret += "INDEX " + mysql->Quote(index->Name) + ", ";
        break;
      }
    }
  }

  if (deletedIndexes.count() > 0)
    for (QStringList::Iterator it = deletedIndexes.begin(); it != deletedIndexes.end(); ++it)
      ret += *it;

  return ret;
}

QString CTableIndexes::getIndexes(bool alter)
{
  QString ret = QString::null;
  for (int i = 0; i < indexName->count(); i++)
  {
    QString txt = indexName->text(i);
    Index *index = Indexes.find(txt);
    QString p = getIndexFields(index);
    txt = mysql->Quote(index->Name);
    if (!p.isNull())
    {
      if (alter)
        ret += "ADD ";
      switch (index->Type)
      {
      case PRIMARY: ret += "PRIMARY KEY ";
        break;
      case INDEX: ret += "INDEX " + txt;
        break;
      case UNIQUE: ret += "UNIQUE " + (index->UniFullIndex ? QString("INDEX ") : QString::null) + txt;
        break;
      case FULLTEXT: ret += "FULLTEXT " + (index->UniFullIndex ? QString("INDEX ") : QString::null)  + txt;
        break;
      }
      ret += "(" + p + "), ";
    }
  }
  return ret;
}

void CTableIndexes::DeleteIndex()
{
  if (g_confirmCritical)
    if ((QMessageBox::information(0, tr("Confirm Delete"), tr("Are you sure you want to Delete Index") +
		      ": \"" + indexName->currentText() + "\"?", tr("&Yes"),tr("&No")) != 0))
      return;  
  Index *index = Indexes.find(indexName->currentText());
  if (!index->isNewIndex)
    deletedIndexes.append("DROP INDEX " + mysql->Quote(index->Name) + ", ");
  Indexes.remove(indexName->currentText());
  int i = indexName->currentItem();
  indexName->removeItem(i);
  indexName->setCurrentItem(i - 1);
  refresh();
}

void CTableIndexes::ChangeType(bool b, IndexType t)
{
  if (b)
  {
    QString current = indexName->currentText();
    Index *index = Indexes.find(current);
    index->Type = t;    
    uniFullIndex->setEnabled(!(t == INDEX || t == PRIMARY));
    int i = indexName->currentItem();
    QPixmap im;
    switch (t)
    {
    case INDEX:
      uniFullIndex->setChecked(false);
      im = getPixmapIcon("mulIcon");
      break;
    case UNIQUE:      
      im = getPixmapIcon("uniIcon");
      break;
    case FULLTEXT:      
      im = getPixmapIcon("ftIcon");
      break;
    case PRIMARY:
      break;
    }
    indexName->changeItem(im, current, i);
  }
}

void CTableIndexes::UniFtToggled(bool b)
{  
  Index *index = Indexes.find(indexName->currentText());
  index->UniFullIndex = b;
}

void CTableIndexes::UniToggled(bool b)
{  
  ChangeType(b, UNIQUE);  
}

void CTableIndexes::IdxToggled(bool b)
{
  ChangeType(b, INDEX);  
}

void CTableIndexes::FtToggled(bool b)
{
  ChangeType(b, FULLTEXT);  
}

void CTableIndexes::IndexChanged(const QString &)
{
  refresh();
}

void CTableIndexes::init()
{
  connect(removeField, SIGNAL(clicked()), this, SLOT(RemoveField()));
  connect(addField, SIGNAL(clicked()), this, SLOT(AddField()));
  connect(addIndex, SIGNAL(clicked()), this, SLOT(AddIndex()));
  connect(deleteIndex, SIGNAL(clicked()), this, SLOT(DeleteIndex()));
  connect(indexName, SIGNAL(activated(const QString &)), this, SLOT(IndexChanged(const QString &)));
  connect(uniqueIndex, SIGNAL(toggled(bool)), this, SLOT(UniToggled(bool)));
  connect(indexIndex, SIGNAL(toggled(bool)), this, SLOT(IdxToggled(bool)));
  connect(fullTextIndex, SIGNAL(toggled(bool)), this, SLOT(FtToggled(bool)));
  connect(uniFullIndex, SIGNAL(toggled(bool)), this, SLOT(UniFtToggled(bool)));
}

CFieldProperties::CFieldProperties(QWidget* parent, CMessagePanel *m, const char* name, WFlags fl)
: QWidget(parent, name, fl)
{
  if (!name)
    setName("CFieldProperties");
  Field = 0;
  AutoIncField = 0;
  messagePanel = m;
  intValidator = new QIntValidator(this);
  intValidator->setBottom(1);  
  floatValidator = new QRegExpValidator(QRegExp("\\d+\\s*,\\s*\\d+"), this);
  setCaption(trUtf8("Field Properties"));
  CFieldPropertiesLayout = new QGridLayout(this, 1, 1, 4, 2, "CFieldPropertiesLayout");
  
  TextLabel1 = new QLabel(this, "TextLabel1");
  TextLabel1->setText(trUtf8("Default"));
  
  CFieldPropertiesLayout->addWidget(TextLabel1, 1, 0);
  
  TextLabel2 = new QLabel(this, "TextLabel2");
  TextLabel2->setText(trUtf8("Length"));
  
  CFieldPropertiesLayout->addWidget(TextLabel2, 0, 0);
  
  TextLabel3 = new QLabel(this, "TextLabel3");
  TextLabel3->setText(trUtf8("Value"));
  
  CFieldPropertiesLayout->addWidget(TextLabel3, 2, 0);
  
  Default = new p_LineEdit(this, "Default");
  QWhatsThis::add(Default, trUtf8("Default value for the Field."));
  
  CFieldPropertiesLayout->addMultiCellWidget(Default, 1, 1, 1, 4);
  
  Value = new p_LineEdit(this, "Value");
  QWhatsThis::add(Value, trUtf8("This stores the values for SET & ENUM Fields.  Separate each field with commas and when using text values, enclose them with single-quotes."));
  
  CFieldPropertiesLayout->addMultiCellWidget(Value, 2, 2, 1, 4);
  
  Length = new p_LineEdit(this, "Length");
  Length->setValidator(intValidator);
  QWhatsThis::add(Length, trUtf8("Maximum Length of the Field.  This field will act as Key Length for BLOB & TEXT Fields."));
  
  CFieldPropertiesLayout->addMultiCellWidget(Length, 0, 0, 1, 4);
  
  Unsigned = new p_CheckBox(this, "Unsigned");
  Unsigned->setText("UNSIGNED");
  QWhatsThis::add(Unsigned, trUtf8("Set the UNSIGNED property for this field.  For more information refere to the MySQL manual."));
  
  CFieldPropertiesLayout->addMultiCellWidget(Unsigned, 3, 3, 0, 1);
  
  Binary = new p_CheckBox(this, "Binary");
  Binary->setText("BINARY");
  QWhatsThis::add(Binary, trUtf8("Set the BINARY property for this field.  For more information refere to the MySQL manual."));
  
  CFieldPropertiesLayout->addMultiCellWidget(Binary, 4, 4, 0, 1);
  
  ZeroFill = new p_CheckBox(this, "ZeroFill");
  ZeroFill->setText("ZEROFILL");
  QWhatsThis::add(ZeroFill, trUtf8("Set the ZEROFILL property for this field.  For more information refere to the MySQL manual."));
  
  CFieldPropertiesLayout->addWidget(ZeroFill, 4, 3);
  
  AutoIncrement = new p_CheckBox(this, "AutoIncrement");
  AutoIncrement->setText("AUTO_INCREMENT");
  QWhatsThis::add(AutoIncrement, trUtf8("Set the AUTO_INCREMENT property for this field.  For more information refere to the MySQL manual."));
  
  CFieldPropertiesLayout->addWidget(AutoIncrement, 3, 3);
  QSpacerItem* spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
  CFieldPropertiesLayout->addItem(spacer, 4, 4);
  QSpacerItem* spacer_2 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
  CFieldPropertiesLayout->addItem(spacer_2, 5, 3);
  QSpacerItem* spacer_3 = new QSpacerItem(50, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);
  CFieldPropertiesLayout->addItem(spacer_3, 3, 2);  
  
  setTabOrder(Length, Default);
  setTabOrder(Default, Value);
  setTabOrder(Value, Unsigned);
  setTabOrder(Unsigned, AutoIncrement);
  setTabOrder(AutoIncrement, Binary);
  setTabOrder(Binary, ZeroFill);

  skip = true;
  connect(Default, SIGNAL(textChanged(const QString &)), this, SLOT(setDefault(const QString &)));
  connect(Value, SIGNAL(textChanged(const QString &)), this, SLOT(setValue(const QString &)));
  connect(Length, SIGNAL(textChanged(const QString &)), this, SLOT(setLength(const QString &)));
  connect(Unsigned, SIGNAL(toggled(bool)), this, SLOT(setUnsigned(bool)));
  connect(Binary, SIGNAL(toggled(bool)), this, SLOT(setBinary(bool)));
  connect(ZeroFill, SIGNAL(toggled(bool)), this, SLOT(setZeroFill(bool)));
  connect(AutoIncrement, SIGNAL(toggled(bool)), this, SLOT(setAutoIncrement(bool)));
  skip = false;
}

void CFieldProperties::setDefault(const QString &s)
{
  if (Field == 0 || skip)
    return;
  Field->Default = s;
}

void CFieldProperties::setValue(const QString &s)
{
  if (Field == 0 || skip)
    return;
  Field->Value = s;
}

void CFieldProperties::setLength(const QString &s)
{
  if (Field == 0 || skip)
    return;
  Field->Length = s;
}

void CFieldProperties::setUnsigned(bool b)
{
  if (Field == 0 || skip)
    return;
  Field->Unsigned = b;
}

void CFieldProperties::setBinary(bool b)
{
  if (Field == 0 || skip)
    return;
  Field->Binary = b;
}

void CFieldProperties::setZeroFill(bool b)
{
  if (Field == 0 || skip)
    return;
  Field->ZeroFill = b;
}

void CFieldProperties::setAutoIncrement(bool b)
{
  if (Field == 0 || skip)
    return;  
  Field->AutoIncrement = b;  
  if (b)
  {
    if (AutoIncField != 0)
    {
      messagePanel->Warning(tr("You can only have one AUTO_INCREMENT field in a Table.  The AUTO_INCREMENT property of field '" + AutoIncField->FieldName + "' was removed."));
      AutoIncField->AutoIncrement = false;
    }
    AutoIncField = Field;
  }
  else  
    AutoIncField = 0;    
}

void CFieldProperties::setFieldType(const QString &t)
{
  skip = true;
  QString type = t.lower().stripWhiteSpace();
  if (type == "real" || type == "double" || type == "float" || type == "decimal" || type == "numeric")
    Length->setValidator(floatValidator);    
  else
    Length->setValidator(intValidator);
  
  if (type == "bit" || type == "bool" || (type.find ("int") != -1) || type == "real" || 
    type == "double" || type == "float" || type == "decimal" || type == "numeric")  //AUTO_INCREMENT, ZEROFILL, UNSIGNED
  {    
    Value->setEnabled(false);
    Binary->setEnabled(false);
    
    Field->Value = QString::null;
    Field->Binary = false;
    
    Length->setEnabled(true);
    Unsigned->setEnabled(true);
    ZeroFill->setEnabled(true);
    AutoIncrement->setEnabled(true);
    Default->setEnabled(true);
  }
  else
    if (type.find("char") != -1) //BINARY
    {
      Value->setEnabled(false);
      Unsigned->setEnabled(false);
      ZeroFill->setEnabled(false);
      AutoIncrement->setEnabled(false);
      
      Field->Value = QString::null;
      Field->Unsigned = false;
      Field->ZeroFill = false;
      Field->AutoIncrement = false;
      
      Length->setEnabled(true);
      Binary->setEnabled(true);
      Default->setEnabled(true);
    }   
    else
      if (type == "enum" || type == "set") //(value1, value2, ...)
      {
        Length->setEnabled(false);
        Unsigned->setEnabled(false);
        Binary->setEnabled(false);
        ZeroFill->setEnabled(false);
        AutoIncrement->setEnabled(false);
        
        Field->Length = QString::null;
        Field->Unsigned = false;
        Field->Binary = false;
        Field->ZeroFill = false;
        Field->AutoIncrement = false;
        
        Value->setEnabled(true);
        Default->setEnabled(true);
      }
      else
        if ((type.find("blob") != -1) || (type.find("text") != -1))
        {          
          Value->setEnabled(false);          
          Default->setEnabled(false);
          Unsigned->setEnabled(false);
          Binary->setEnabled(false);
          ZeroFill->setEnabled(false);
          AutoIncrement->setEnabled(false);
          
          Field->Unsigned = false;
          Field->Binary = false;
          Field->ZeroFill = false;
          Field->AutoIncrement = false;
          Field->Default = QString::null;
          Field->Value = QString::null;
          
          Length->setEnabled(true);  //used for indexes
        }
        else          
        {
          Value->setEnabled(false);
          Length->setEnabled(false);
          Unsigned->setEnabled(false);
          Binary->setEnabled(false);
          ZeroFill->setEnabled(false);
          AutoIncrement->setEnabled(false);
          
          Field->Unsigned = false;
          Field->Binary = false;
          Field->ZeroFill = false;
          Field->AutoIncrement = false;
          Field->Value = QString::null;
          Field->Length = QString::null;
          
          bool b = (type != "timestamp");
          Default->setEnabled(b);
          if (!b)
            Field->Default = QString::null;
        }
  skip = false;
}

void CFieldProperties::setField(p_FieldInfo *field)
{
  Field = field;
  setFieldType(Field->Type);
  skip = true;
  Default->setText(Field->Default);
  Value->setText(Field->Value);
  Length->setText(Field->Length);  
  Unsigned->setChecked(Field->Unsigned);
  Binary->setChecked(Field->Binary);
  ZeroFill->setChecked(Field->ZeroFill);
  AutoIncrement->setChecked(Field->AutoIncrement);
  skip = false;
}

CFieldProperties::~CFieldProperties()
{
}

CComboFieldItem::CComboFieldItem(QTable * table, int fieldid, const QStringList & list, bool editable)
: QComboTableItem(table, list, editable)
{
  Field = new p_FieldInfo(fieldid);
}

CComboFieldItem::CComboFieldItem(QTable * table, p_FieldInfo *f, const QStringList & list, bool editable)
: QComboTableItem(table, list, editable)
{
  Field = new p_FieldInfo(*f);
}

CComboFieldItem::CComboFieldItem(QTable * table, p_FieldInfo *f, int fieldid, const QStringList & list, bool editable)
: QComboTableItem(table, list, editable)
{
  Field = new p_FieldInfo(*f);
  Field->setID(fieldid);
}

CComboFieldItem::~CComboFieldItem()
{
  delete Field;
}  

CTableWindow::CTableWindow(QWidget* parent, CMySQLConnection *m, CDatabaseTreeItem *item, const QString &tablename)
: CMyWindow(parent, "CTableWindow", WDestructiveClose, !g_isMDI, true)
{
  tableItem = item;
  createForm(m, tablename);
}

CTableWindow::CTableWindow(QWidget* parent, CMySQLConnection *m, const QString &tablename, QObject *o, const char *slot)
: CMyWindow(parent, "CTableWindow", WDestructiveClose, !g_isMDI, true)
{
  tableItem = 0;
  connect (this, SIGNAL(doRefresh()), o, slot);
  createForm(m, tablename);
}

void CTableWindow::refreshCaller()
{  
  if (tableItem != 0)
  {
    if (isAlterTable)    
      tableItem->listView()->setCurrentItem(tableItem);      
    tableItem->refresh();
  }
  else
    emit doRefresh();
}

void CTableWindow::setWindowCaption()
{
  QString c;
  if (isAlterTable)
    c = tr("Editing Table") + " '" + tableName + "' ";
  else
    c = tr("Creating Table") + " ";
  c += tr("in database") + " '" + mysql->getDatabaseName() + "' [" + mysql->getConnectionName() + "]"; 
  setCaption(c);
}

void CTableWindow::createForm(CMySQLConnection *m, const QString &tablename)
{
  mysql = new CMySQLConnection(m->getConnectionName(), messagePanel());  
  mysql->connect(m->getDatabaseName());
  isAlterTable = !tablename.isNull();
  pasteField = 0;
  hasPaste = false;
  hasModified = false;
  tableName = tablename;
  
  pkIcon = getPixmapIcon("pkIcon");
  emptyPkIcon = getPixmapIcon("emptyPkIcon");
  pkIconSet.setPixmap(pkIcon, QIconSet::Automatic);

  FieldTypes = new QStringList();
  DeletedFields = new QStringList();
  bool initOk = loadSyntaxSection(FieldTypes, 9);
  defaultFieldType = FieldTypes->findIndex("varchar");
  if (defaultFieldType == -1)
    defaultFieldType = 0;
  defaultFieldLength = "100";
  if (initOk)
    messagePanel()->Information(tr("Field Types loaded correctly."));
  else
    messagePanel()->Critical(tr("Could not load Field Types.  Please select the location of the 'syntax.txt' file in 'Options->General->Sql Editor' and re-open this dialog."));  

  (void)statusBar();

  setIcon(getPixmapIcon("applicationIcon"));  

  setCentralWidget(new QWidget(this, "qt_central_widget"));
  CTableWindowLayout = new QGridLayout(centralWidget(), 1, 1, 4, 2, "CTableWindowLayout"); 
  
  Fields = new QTable(centralWidget(), "Fields");
  Fields->setNumCols(3);
  Fields->setSelectionMode(QTable::MultiRow);  
    
  CTableWindowLayout->addWidget(Fields, 0, 0);
  
  Tab = new QTabWidget(centralWidget(), "Tab");
  Tab->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)7, (QSizePolicy::SizeType)1, 0, 0, Tab->sizePolicy().hasHeightForWidth()));
  Tab->setMinimumSize(QSize(0, 190));
  
  fieldPropertiesTab = new CFieldProperties(Tab, messagePanel(), "fieldPropertiesTab");
  Tab->insertTab(fieldPropertiesTab, fieldPropertiesTab->caption());
  
  indexesTab = new CTableIndexes(Fields, messagePanel(), Tab, "indexesTab");
  indexesTab->setMySQL(mysql);
  Tab->insertTab(indexesTab, indexesTab->caption());
  
  tablePropertiesTab = new CTblProperties(Tab, mysql, "tablePropertiesTab");
  Tab->insertTab(tablePropertiesTab, tablePropertiesTab->caption());

  alterTableOptionsTab = new CAlterTableOptions(Tab, Fields, mysql, "alterTableOptions");
  Tab->insertTab(alterTableOptionsTab, alterTableOptionsTab->caption());
  
  CTableWindowLayout->addWidget(Tab, 1, 0);
  
  fileSaveAction = new QAction(this, "fileSaveAction");
  fileSaveAction->setIconSet(QIconSet(getPixmapIcon("saveIcon")));
  fileSaveAction->setText(trUtf8("Save"));
  fileSaveAction->setMenuText(trUtf8("&Save"));
  fileSaveAction->setAccel(0);

  fileSaveAsAction = new QAction(this, "fileSaveAsAction");
  fileSaveAsAction->setText(trUtf8("Save As"));
  fileSaveAsAction->setMenuText(trUtf8("Save &As..."));
  fileSaveAsAction->setAccel(0);

  fileCloseAction = new QAction(this, "fileCloseAction");
  fileCloseAction->setText(trUtf8("Close"));
  fileCloseAction->setIconSet(QIconSet(getPixmapIcon("closeIcon")));  
  fileCloseAction->setMenuText(trUtf8("&Close"));
  fileCloseAction->setAccel(0);

  fileReloadAction = new QAction(this, "fileReloadAction");
  fileReloadAction->setText(trUtf8("Reload"));
  fileReloadAction->setIconSet(QIconSet(getPixmapIcon("refreshTablesIcon")));
  fileReloadAction->setMenuText(trUtf8("&Reload"));
  fileReloadAction->setAccel(0);

  showMessagesAction = new QAction(this, "showMessagesAction");
  Q_CHECK_PTR(showMessagesAction);
  showMessagesAction->setText(tr("Show Messages Panel", ""));  
  showMessagesAction->setMenuText(tr("Show &Messages Panel", ""));
  showMessagesAction->setAccel(0);
  showMessagesAction->setToggleAction(true);
  showMessagesAction->setOn(messageWindow()->isVisible());

  tableInsertAction = new QAction(this, "tableInsertAction");
  tableInsertAction->setText(trUtf8("Insert Row"));
  tableInsertAction->setMenuText(trUtf8("&Insert Row"));
  tableInsertAction->setIconSet(QIconSet(getPixmapIcon("insertRowIcon")));
  tableInsertAction->setAccel(0);

  tableDeleteAction = new QAction(this, "tableDeleteAction");
  tableDeleteAction->setText(trUtf8("Delete Row"));
  tableDeleteAction->setMenuText(trUtf8("&Delete Row"));
  tableDeleteAction->setIconSet(QIconSet(getPixmapIcon("deleteRowIcon")));
  tableDeleteAction->setAccel(0);

  tablePrimaryKeyAction = new QAction(this, "tablePrimaryKeyAction");
  tablePrimaryKeyAction->setText(trUtf8("Add Primary Key"));
  tablePrimaryKeyAction->setMenuText(trUtf8("Add Primary Key"));
  tablePrimaryKeyAction->setIconSet(QIconSet(pkIcon));
  tablePrimaryKeyAction->setAccel(0);

  editCopyAction = new QAction(this, "editCopyAction");
  editCopyAction->setText(trUtf8("Copy"));
  editCopyAction->setMenuText(trUtf8("&Copy"));
  editCopyAction->setIconSet(QIconSet(getPixmapIcon("copyIcon")));
  editCopyAction->setAccel(0);

  editPasteAction = new QAction(this, "editPasteAction");
  editPasteAction->setText(trUtf8("Paste"));
  editPasteAction->setMenuText(trUtf8("&Paste"));
  editPasteAction->setIconSet(QIconSet(getPixmapIcon("pasteIcon")));
  editPasteAction->setAccel(0);
  editPasteAction->setEnabled(false);

  fileBar = new QToolBar(tr("File"), this, DockTop);  
  fileSaveAction->addTo(fileBar);  
  fileReloadAction->addTo(fileBar);
  fileBar->addSeparator();
  editCopyAction->addTo(fileBar);
  editPasteAction->addTo(fileBar);

  tableBar = new QToolBar(tr("Table"), this, DockTop);
  tablePrimaryKeyAction->addTo(tableBar);
  tableBar->addSeparator();
  tableInsertAction->addTo(tableBar);
  tableDeleteAction->addTo(tableBar);

  menubar = new QMenuBar(this, "menubar");

  fileMenu = new QPopupMenu(this);
  fileSaveAction->addTo(fileMenu);
  fileSaveAsAction->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileReloadAction->addTo(fileMenu);
  fileMenu->insertSeparator();
  showMessagesAction->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileCloseAction->addTo(fileMenu);
  menubar->insertItem(trUtf8("&File"), fileMenu);

  editMenu = new QPopupMenu(this);
  editCopyAction->addTo(editMenu);
  editPasteAction->addTo(editMenu);
  menubar->insertItem(trUtf8("&Edit"), editMenu);

  tableMenu = new QPopupMenu(this);
  tablePrimaryKeyAction->addTo(tableMenu);
  tableMenu->insertSeparator();
  tableInsertAction->addTo(tableMenu);
  tableDeleteAction->addTo(tableMenu);
  menubar->insertItem(trUtf8("&Table"), tableMenu);
  init(initOk);
  myResize(550, 500);
  autoPlace();
}

CTableWindow::~CTableWindow()
{  
  delete mysql;
  delete FieldTypes;
  delete DeletedFields;
  if (hasPaste)         
    delete pasteField;  
}

void CTableWindow::addPrimaryIcon(bool add, int row)
{  
  if (add)
    Fields->verticalHeader()->setLabel(row, pkIconSet, QString::null);
  else
    Fields->verticalHeader()->setLabel(row, QIconSet(), QString::null);
}

bool CTableWindow::checkDuplicateNames(const QString &txt, int row, int col)
{
  bool err = false;
  for (int i = 0; i < (int) Fields->numRows(); i++)    
    if ((Fields->text(i,0) == txt) && (Fields->item(i, 2)->row() != row))
    {
      messagePanel()->Warning(tr("You already have a Field Name called") + " '" + txt + "'");
      Fields->editCell(row, col);
      err = true;
      break;
    }
  return err;
}

void CTableWindow::fieldsValueChanged(int row, int col)
{
  qDebug("CTableWindow::fieldsValueChanged()");
  p_FieldInfo *field = ((CComboFieldItem *)Fields->item(row, 2))->Field;
  switch (col)
  {
  case 0: field->FieldName = Fields->text(row,col);
          checkDuplicateNames(field->FieldName, row, col);
    break;
  case 1: field->Null = ((QCheckTableItem *) Fields->item(row, col))->isChecked();
    break;
  case 2: 
    {
      QString t = Fields->text(row, col);
      if (field->Type != t)
      {
        field->Length = QString::null;
        field->Type = t;
      }
      fieldPropertiesTab->setField(field);
    }    
    break;
  }
  hasModified = true;
  emit valueChanged();
  qDebug("CTableWindow::fieldsValueChanged() - end");
}

void CTableWindow::fieldsClicked(int row, int, int, const QPoint &)
{
  qDebug("CTableWindow::fieldsClicked()");
  p_FieldInfo *field = ((CComboFieldItem *)Fields->item(row, 2))->Field;   
  if (field->Type.isNull())
    field->Type = Fields->text(row, 2);    
  fieldPropertiesTab->setField(field);

  if (!Fields->verticalHeader()->iconSet(row)->isNull())
  {    
    tablePrimaryKeyAction->setText(trUtf8("Drop Primary Key"));
    tablePrimaryKeyAction->setMenuText(trUtf8("Dr&op Primary Key"));
    tablePrimaryKeyAction->setIconSet(QIconSet(emptyPkIcon));
  }
  else
  {   
    tablePrimaryKeyAction->setText(trUtf8("Add Primary Key"));
    tablePrimaryKeyAction->setMenuText(trUtf8("Add Primary &Key"));
    tablePrimaryKeyAction->setIconSet(QIconSet(pkIcon));
  }
  qDebug("CTableWindow::fieldsClicked() - end");
}

void CTableWindow::deleteRow()
{
  qDebug("CTableWindow::deleteRow()");
  int currentRow = Fields->currentRow();
  int currentColumn = Fields->currentColumn();
  if (currentRow == -1 || currentColumn == -1)
    return;
  int numRows = Fields->numRows();
  int min = numRows - 1;
  QIntDict<bool> *selected = new QIntDict<bool>(numRows);
  selected->setAutoDelete(true);
  for (int j = 0; j < numRows; j++)
    if (Fields->isRowSelected(j))
      selected->insert(j, new bool(true));
  if (selected->isEmpty())
    selected->insert(currentRow, new bool(true));
  Fields->clearSelection();

  if ((numRows - selected->count()) < 1)
    messagePanel()->Warning(tr("You must have at least one field in the table"));
  else
  {
    hasModified = true;
    int factor = 0;
    QIntDictIterator<bool> it(*selected);
    for (; it.current(); ++it)
    {
      int row = it.currentKey() - factor++;
      if (isAlterTable)
      {
        p_FieldInfo *field = ((CComboFieldItem *)Fields->item(row, 2))->Field;
        if (!field->newField)
          DeletedFields->append(field->originalFieldName);
      }
      if (row < min)
        min = row;
      Fields->removeRow(row);      
    }
  }  
  delete selected;
  refreshInternalIdx();
  min = min < 1 ? 0 : min - 1;  
  Fields->setCurrentCell(min, currentColumn);
  fieldsClicked(min, 0, Qt::LeftButton, QPoint(0,0));
  Fields->adjustRow(0);
  emit valueChanged();
  qDebug("CTableWindow::deleteRow() - end");
}

void CTableWindow::refreshInternalIdx()
{
  for (int i = 0; i < (int) Fields->numRows(); i++)
  {
    p_FieldInfo *f = ((CComboFieldItem *)Fields->item(i, 2))->Field;
    f->setID(i);
  }
}

void CTableWindow::insert(p_FieldInfo *f)
{
  qDebug("CTableWindow::insert(p_FieldInfo *)");
  int currentRow = Fields->currentRow();
  int currentColumn = Fields->currentColumn();
  if (currentRow == -1 || currentColumn == -1)
    return;  
  Fields->insertRows(currentRow);
  CComboFieldItem *combo;  
  QCheckTableItem *check = new QCheckTableItem(Fields, QString::null);
  if (f != 0)  //paste insert
  {
    hasModified = true;
    combo = new CComboFieldItem(Fields, f, *FieldTypes, false);
    check->setChecked(f->Null);
    combo->setCurrentItem(f->Type);
    Fields->setText(currentRow, 0, f->FieldName);
  }
  else  //plain insert
  {
    combo = new CComboFieldItem(Fields, currentRow, *FieldTypes, false);  
    combo->setCurrentItem(defaultFieldType);
    combo->Field->Length = "100";
  }
  combo->Field->newField = true;
  Fields->setItem(currentRow, 1, check);
	Fields->setItem(currentRow, 2, combo);
  refreshInternalIdx();
  Fields->setCurrentCell(currentRow, currentColumn);  
  fieldsClicked(currentRow, 0, Qt::LeftButton, QPoint(0,0));
  qDebug("CTableWindow::insert(p_FieldInfo *) - end");
}

void CTableWindow::insertRow()
{
  insert(0);  
}

void CTableWindow::reset()
{
  qDebug("CTableWindow::reset()");
  setWindowCaption();
  QIntDict<p_FieldInfo> *fieldsDict = 0;
  QStringList keys;
  fileReloadAction->setEnabled(isAlterTable);
  fileSaveAsAction->setEnabled(isAlterTable);
  indexesTab->setIsAlterTable(isAlterTable);
  Tab->setTabEnabled(alterTableOptionsTab, isAlterTable);
  DeletedFields->clear();  
  Fields->setRowMovingEnabled(mysql->getMySQLServerVersion().major > 3 || !isAlterTable);
  Fields->setNumRows(0);
  indexesTab->clear();
  if (isAlterTable)
  {
    alterTableOptionsTab->reset();    
    CMySQLQuery query(mysql);
    keys.clear();
    query.exec_static("SET SQL_QUOTE_SHOW_CREATE = 1");
    QString qry = "SHOW CREATE TABLE " + mysql->Quote(tableName);
    QString createQuery = QString::null;
    if (query.exec(qry, false))
    {     
      query.next();
      createQuery = query.Row(1);
    }

    if (createQuery.isEmpty())
    {
      messagePanel()->Critical(tr("Failed to execute") + ": " + qry);
      return;
    }
    tablePropertiesTab->tableName->setText(tableName);
    QStringList lines = QStringList::split("\n", createQuery);
    uint cnt = 0;
    fieldsDict = new QIntDict<p_FieldInfo>(lines.count());
    fieldsDict->setAutoDelete(true);
    for (QStringList::Iterator it = lines.begin(); it != lines.end(); ++it)  //parse the query
    {
      if (it == lines.begin())  //create definition
        continue;
      else
        if (it == lines.fromLast())  //table options
        {
          QStringList opt = QStringList::split(" ", *it);
          for (QStringList::Iterator op = opt.begin(); op != opt.end(); ++op)  //parse table options
          {
            if ((*op) == ")")
              continue;
            int sep = (*op).find('=');
            QString option = (*op).left(sep).lower();
            QString value = (*op).mid(sep+1);
            
            if (option == "min_rows")
              tablePropertiesTab->minRows->setText(value);
            else
              if (option == "max_rows")
                tablePropertiesTab->maxRows->setText(value);
              else
                if (option == "avg_row_length")
                  tablePropertiesTab->avgRowLength->setText(value);
                else
                  if (option == "pack_keys")
                    tablePropertiesTab->packKeys->setCurrentText(value);
                  else
                    if (option == "checksum")
                      tablePropertiesTab->checksum->setCurrentText(value);
                    else
                      if (option == "delay_key_write")
                        tablePropertiesTab->delayKeyWrite->setCurrentText(value);
                      else
                        if (option == "row_format")
                          tablePropertiesTab->rowFormat->setCurrentText(value);
                        else
                          if (option == "auto_increment")
                            tablePropertiesTab->autoIncrement->setText(value);
                          else
                            if (option == "comment")
                              tablePropertiesTab->comment->setText(value.mid(1, value.length() - 2));
                            else
                              if (option == "type")
                                for (int x = 0; x < tablePropertiesTab->tableType->count(); x++)
                                  if (tablePropertiesTab->tableType->text(x).lower() == value.lower())
                                  {
                                    tablePropertiesTab->tableType->setCurrentItem(x);
                                    break;
                                  }
          }
        }
        else  //fields or indexes
        {
          QString line = (*it).stripWhiteSpace();          
          if (line.right(1) == ",")
            line = line.left(line.length() - 1);
          if (line.at(0) == '`')  //its a field
          {
            p_FieldInfo *f = new p_FieldInfo(cnt);
            line = line.mid(1);

            int p = line.find('`');
            f->FieldName = line.left(p);

            line = line.mid(p + 1).stripWhiteSpace();

            bool hasquote = false;
            bool hasparen = false;
            for (uint x = 0; x < line.length(); x++)
            {
              QChar c = line.at(x);
              if (c == '\'')
                hasquote = !hasquote;
              else
                if (c == '(' && !hasquote)
                  hasparen = true;
                else
                  if (c == ')' && !hasquote)
                    hasparen = false;
              if (!hasquote && !hasparen && c == ' ')
              {
                p = x;
                break;
              }
            }
            f->Type = line.left(p);
            line = line.mid(p + 1).stripWhiteSpace();
            p = f->Type.find('(');
            if (p != -1) // type has length or value;
            {
              f->Length = f->Type.mid(p + 1);
              f->Length = f->Length.left(f->Length.findRev(')'));
              f->Type = f->Type.left(p);
              if (f->Type.lower() == "enum" || f->Type.lower() == "set")
              {
                f->Value = f->Length;
                f->Length = QString::null;
              }
            }
            p = line.find("'");
            if (p != -1) //has default ... auto_increment fields can't have default !
            {
              f->Default = line.mid(p + 1);
              line = line.left(p);
              f->Default = f->Default.left(f->Default.findRev('\''));
              f->Default = f->Default.replace(QRegExp("''"), "'");
            }
            line = line.lower();
            f->Unsigned = (line.find("unsigned") != -1);
            f->ZeroFill = (line.find("zerofill") != -1);
            f->AutoIncrement = (line.find("auto_increment") != -1);
            f->Binary = (line.find("binary") != -1);
            f->Null = (line.find("not null") == -1);
            fieldsDict->insert(cnt, f);
            cnt++;
          }
          else  //must be an index
            keys.append(*it);  //keys must be added once all the fields are added to the grid.
        }
    }  //end parse query
    Fields->setNumRows(cnt);
    fileReloadAction->setEnabled(true);
    fileSaveAsAction->setEnabled(true);    
  }
  else  //if (isAlterTable)  
    Fields->setNumRows(50);  

  if (hasPaste)
  {
    delete pasteField;
    pasteField = 0;
    hasPaste = false;
  }

  Fields->horizontalHeader()->setLabel(0, tr("Field Name"), 210);
  Fields->horizontalHeader()->setLabel(1, tr("Allow NULL"));
  Fields->horizontalHeader()->setLabel(2, tr("Data Type"));

  for (int i = 0; i < Fields->numRows(); ++i)
  {
    Fields->verticalHeader()->setLabel(i, QString::null);
    CComboFieldItem * combo = 0;    
    QCheckTableItem *check = new QCheckTableItem(Fields, QString::null);
    if (isAlterTable)
    {      
      combo = new CComboFieldItem(Fields, fieldsDict->find(i), i, *FieldTypes, false);
      combo->setCurrentItem(combo->Field->Type);      
      check->setChecked(combo->Field->Null);
      Fields->setText(i, 0, combo->Field->FieldName);
      combo->Field->originalFieldName = combo->Field->FieldName;  //ALTER TABLE CHANGE originalFieldName FieldName ...
    }
    else
    {
      combo = new CComboFieldItem(Fields, i, *FieldTypes, false);
	    combo->setCurrentItem(defaultFieldType);
      combo->Field->Length = defaultFieldLength;
    }
    Fields->setItem(i, 1, check);
	  Fields->setItem(i, 2, combo);
  }
  if (isAlterTable)
  {
    alterTableOptionsTab->refresh();
    for (QStringList::Iterator idx = keys.begin(); idx != keys.end(); ++idx)
    {
      QString k = (*idx).stripWhiteSpace();
      CTableIndexes::IndexType type;
      QString t = k.left(k.find(' ')).lower();
      QString name;
      if (t == "primary")
      {
        type = CTableIndexes::PRIMARY;
        name = "PRIMARY";
        k = k.mid(k.find('(') + 1);
      }
      else
      {
        if (t == "unique")
          type = CTableIndexes::UNIQUE;
        else
          if (t == "key")
            type = CTableIndexes::INDEX;
          else
            type = CTableIndexes::FULLTEXT;
        k = k.mid(k.find('`') + 1);        
        name = k.left(k.find('`'));
        k = k.mid(k.find('(') + 1);
      }      
      k = k.left(k.findRev(')'));
      QStringList flds = QStringList::split(",", k);
      
      for (QStringList::Iterator fld = flds.begin(); fld != flds.end(); ++fld)
      {
        QString f = QString::null;
        QString len = QString::null;
        int cnt = 0;
        for (uint n = 0; n < (*fld).length(); n++)
        {
          QChar c = (*fld).at(n);
          if (c != '`')
          {
            if (cnt < 2)
              f += c;
            else
              if (c != '(' && c != ')')
                len += c;
          }
          else
            cnt++;
        }
        if (!len.isEmpty())
          for (int j = 0; j < Fields->numRows(); j++)
          {
            CComboFieldItem *c = (CComboFieldItem *)Fields->item(j, 2);
            if (c->Field->FieldName == f)
            {
              c->Field->Length = len;
              break;
            }
          }
        (*fld) = f;
      }
      indexesTab->Add(name, type, flds);
    }
    indexesTab->indexName->setCurrentItem(0);  
    delete fieldsDict;
  }
  indexesTab->refresh();
  Fields->setCurrentCell(0, 0);
  Fields->adjustRow(0); 
  fieldsClicked(0, 0, Qt::LeftButton, QPoint(0,0));
  qDebug("CTableWindow::reset() - end");
}

void CTableWindow::copy()
{
  if (hasPaste)
  {
    delete pasteField;
    pasteField = 0;
  }
  pasteField = new p_FieldInfo(*((CComboFieldItem *)Fields->item(Fields->currentRow(), 2))->Field);
  if (pasteField->AutoIncrement)
    pasteField->AutoIncrement = false;
  hasPaste = true;
  editPasteAction->setEnabled(true);
}

void CTableWindow::paste()
{
  if (hasPaste)
  {
    insert(pasteField);
    emit valueChanged();    
  }
}

void CTableWindow::setPk()
{
  bool isPrimaryKey = !Fields->verticalHeader()->iconSet(Fields->currentRow())->isNull();
  setPrimaryKey(Fields->currentRow(), isPrimaryKey);
}

void CTableWindow::setPrimaryKey(int row, bool b)
{
  QString txt = Fields->text(row, 0);
  if (!txt.isEmpty())
  {
    if (!b)
      indexesTab->singleAdd("PRIMARY", txt);
    else
      indexesTab->singleRemove("PRIMARY", txt);
  }
  fieldsClicked(row, 0, Qt::LeftButton, QPoint(0,0));
}

void CTableWindow::showContextMenu(int, int, const QPoint &pos)
{
  QPopupMenu *menu = new QPopupMenu();
  Q_CHECK_PTR(menu);
  int row = Fields->currentRow();  
  bool isPrimaryKey = !Fields->verticalHeader()->iconSet(row)->isNull();
  if (isPrimaryKey)
    menu->insertItem(emptyPkIcon, tr("Drop Primary Key"), 0);
  else
    menu->insertItem(pkIcon, tr("Add Primary Key"), 0);
  menu->insertSeparator();
  menu->insertItem(getPixmapIcon("insertRowIcon"), tr("Insert Row"), 1);
  menu->insertItem(getPixmapIcon("deleteRowIcon"), tr("Delete Row"), 2);
  menu->insertSeparator();
  menu->insertItem(getPixmapIcon("copyIcon"), tr("Copy"), 3);
  menu->insertItem(getPixmapIcon("pasteIcon"), tr("Paste"), 4);
  menu->insertSeparator();
  menu->insertItem(getPixmapIcon("refreshTablesIcon"), tr("Reload"), 5);
  
  menu->setItemEnabled(3, hasPaste);  
  menu->setItemEnabled(4, isAlterTable);

  int res = menu->exec(pos);
  delete menu;
  switch (res)
  {
  case 0: setPrimaryKey(row, isPrimaryKey);    
    break;
  case 1: insertRow();
    break;
  case 2: deleteRow();
    break;
  case 3: copy();
    break;
  case 4: paste();
    break;
  case 5: reset();
    break;
  }  
}

void CTableWindow::closeEvent(QCloseEvent * e)
{
  if (hasModified && g_confirmCritical) 
  {
    if ((QMessageBox::information(0, tr("Save Changes"), tr("Do you want to save your changes ?"),
      tr("&Yes"), tr("&No")) == 0))
      if (!save())        
      {
        e->ignore();
        return;
      }
  }  
  e->accept();
  CMyWindow::closeEvent(e);
}

bool CTableWindow::exec(const QString &qry)
{  
  CMySQLQuery *query = new CMySQLQuery(mysql);
  bool ret = query->exec_static(qry);	
  delete query;
  return ret;
}

QString CTableWindow::getFieldSpecs(p_FieldInfo *field, QStringList &dups, bool &ok)
{
  qDebug("CTableWindow::getFieldSpecs()");
  QString sql = QString::null;  
  if (!field->FieldName.isEmpty() && !field->FieldName.isNull())
  {    
    if (dups.findIndex(field->FieldName) != -1)  //check for duplicate names
    {
      messagePanel()->Critical(tr("You already have a Field Name called") + " '" + field->FieldName + "'");
      ok = false;
      qDebug("CTableWindow::getFieldSpecs() - duplicate field error");
      return QString::null;      
    }
    else
      dups.append(field->FieldName);
    QString type = field->Type.lower();
    sql += mysql->Quote(field->FieldName) + " ";
    sql += field->Type;      
    if (type == "varchar" && field->Length.isEmpty())  //varchar requires length to be defined
    {
      messagePanel()->Warning(tr("All VARCHAR fields must have Length defined...Using the default field length."));
      field->Length = defaultFieldLength;
    }
    if (!field->Length.isEmpty())
    {
      if ((type.find("blob") == -1) && (type.find("text") == -1))
      {
        field->Length = field->Length.stripWhiteSpace().simplifyWhiteSpace();        
        sql += "(" + field->Length + ")";
      }
    }
    else
    {
      if (type == "enum" || type == "set")
      {
        if (!field->Value.isEmpty())
        {          
          QStringList valueList(QStringList::split(",", field->Value, true));
          field->Value = QString::null;
          for (QStringList::Iterator it = valueList.begin(); it != valueList.end(); ++it)
          {
            QString value = (*it).stripWhiteSpace();
            if (!value.isEmpty())
            {
              QChar c = value.at(0);
              if ((c != '\'' && c != '\"') || ((c == '\'' || c == '\"') && value.at(value.length() - 1) != c))
                value = "'" + mysql->escape(value).local8Bit() + "'";
              field->Value += value + ",";
            }
            else
              field->Value += "'',";
          }            
          field->Value = field->Value.left(field->Value.length() - 1);
          sql += "(" + field->Value + ")";            
        }
        else
        {
          messagePanel()->Critical(tr("All ENUM and SET fields must have a Value separated by commas."));
          ok = false;
          qDebug("CTableWindow::getFieldSpecs() - ENUM/SET error");
          return QString::null;
        }
      }        
    }
    sql += " ";
    
    if (field->Unsigned)
      sql += "UNSIGNED ";
    
    if (field->ZeroFill)
      sql += "ZEROFILL ";
    
    if (field->Binary)
      sql += "BINARY ";
    
    if (!field->Null)
      sql += "NOT NULL ";
    
    bool skipDefault = false;

    if (field->Type == "enum" || field->Type == "set")
    {
      if (field->Default.isEmpty())
        skipDefault = true;
    }
    else
      if ( field->Type.find("blob") != -1 || field->Type.find("text") != -1 || field->Type == "timestamp" || field->AutoIncrement)
        skipDefault = true;

    if (!skipDefault)
    {
      sql += "DEFAULT '";    
      if (!field->Default.isEmpty())
        sql += mysql->escape(field->Default).local8Bit();    
      sql += "' ";
    }

    if (field->AutoIncrement)
      sql += "AUTO_INCREMENT ";    
  }
  ok = true;
  qDebug("CTableWindow::getFieldSpecs() - end");
  return sql;
}

bool CTableWindow::createTable(const QString &tblName)
{
  qDebug("CTableWindow::createTable()");
  tableName = tblName;
  QString sql = "CREATE TABLE " + mysql->Quote(tblName) + " (";  
  bool ok = false;
  bool hasFields = false;
  QStringList dups;
  dups.clear();
  for (int i = 0; i < Fields->numRows(); i++)
  {
    p_FieldInfo *field = ((CComboFieldItem *)Fields->item(i, 2))->Field;
    QString s = getFieldSpecs(field, dups, ok);
    if (!ok)
      break;
    else    
      if (!s.isNull())
      {
        hasFields = true;
        sql += s + ", ";
      }
  }
  if (ok)
  {
    if (hasFields)
    {
      sql += indexesTab->getIndexes();
      sql = sql.left(sql.length() - 2);
      sql += ") " + tablePropertiesTab->getTableProperties();
      ok = exec(sql);
    }
    else   
    {
      messagePanel()->Critical(tr("You must have at least one field in the table"));
      ok = false;
    }
  }
  return ok;
  qDebug("CTableWindow::createTable() - end");
}


bool CTableWindow::alterTable()
{
  qDebug("CTableWindow::alterTable()");
  QString sql = "ALTER TABLE ";
  if (alterTableOptionsTab->ignore->isChecked())
    sql += "IGNORE ";
  sql += mysql->Quote(tableName) + " ";
  bool ok = false;
  bool hasFields = false;
  bool isFirst = true;
  QStringList dups;
  dups.clear();
  QString pos;
  QString aboveFieldName = QString::null;
  int ver = mysql->getMySQLServerVersion().major;
  
  sql += indexesTab->getDropIndexes();
  for (QStringList::Iterator it = DeletedFields->begin(); it != DeletedFields->end(); ++it)
    sql += "DROP " + *it + ", ";  

  for (int i = 0; i < Fields->numRows(); i++)
  {    
    p_FieldInfo *field = ((CComboFieldItem *)Fields->item(i, 2))->Field;
    QString s = getFieldSpecs(field, dups, ok);   
    if (!ok)
      break;
    else
      if (!s.isNull())
      {
        hasFields = true;
        if (field->newField)
          sql += "ADD ";
        else        
          sql += "CHANGE " + mysql->Quote(field->originalFieldName) + " ";

        sql += s;

        if (ver > 3 || field->newField)
        {
          if (isFirst)
          {
            sql += "FIRST";
            isFirst = false;
          }
          else
            sql += "AFTER " + aboveFieldName;
        }        
        sql += ", ";
        aboveFieldName = mysql->Quote(field->FieldName);
      }
  }  
  if (ok)
  {
    if (hasFields)
    {
      sql += indexesTab->getIndexes(true);      
      sql += tablePropertiesTab->getTableProperties();
      sql += alterTableOptionsTab->getAlterTableOptions();
      QString t = tablePropertiesTab->tableName->text().stripWhiteSpace();
      if (t.isEmpty())
      {
        messagePanel()->Warning(tr("The original Table Name will be used"));
        t = tableName;
      }
      else
        if (t != tableName) 
          sql += ", RENAME " + mysql->Quote(t);        
      if (ok = exec(sql))
        tableName = t;
    }
    else   
    {
      messagePanel()->Critical(tr("You must have at least one field in the table"));
      ok = false;
    }
  }  
  qDebug("CTableWindow::alterTable() - end");  
  return ok;
}


bool CTableWindow::save()
{
  qDebug("CTableWindow::save()");
  bool ok = false;
  if (!isAlterTable)
  {
    QString tblName = tablePropertiesTab->tableName->text().stripWhiteSpace();
    if (tblName.isEmpty())
    {
      tblName = QInputDialog::getText(tr("Table Name"), tr("Please enter the Table Name"), QLineEdit::Normal, QString::null, &ok);
      tblName = tblName.stripWhiteSpace();
      if (!ok || tblName.isEmpty())
      {
        qDebug("CTableWindow::save() error !ok || tblName.isEmpty()");
        return false;
      }
      else
        tablePropertiesTab->tableName->setText(tblName);
    }
    ok = createTable(tblName);
    if (ok)
      messagePanel()->Information(tr("Table created successfully."));
  }
  else
  {
    ok = alterTable();
    if (ok)
    {
      messagePanel()->Information(mysql->getInfo());
      messagePanel()->Information(tr("Table modified successfully."));
    }
  }
  if (ok)
  {
    isAlterTable = true;
    hasModified = false;
    refreshCaller();
    reset();
  }
  qDebug("CTableWindow::save() - end");
  return ok;
}

void CTableWindow::saveAs()
{
  qDebug("CTableWindow::saveAs()");
  hasModified = false;
  bool ok = false;  
  QString tblName = QInputDialog::getText(tr("Save Table As"), tr("Please enter the new Table Name"), QLineEdit::Normal, QString::null, &ok);
  tblName = tblName.stripWhiteSpace();
  if (ok)  
    if (createTable(tblName))
    {
      messagePanel()->Information(tr("Table created successfully."));
      hasModified = false;
      isAlterTable = true;
      refreshCaller();
      reset();
    }
  qDebug("CTableWindow::saveAs() - end");
}

void CTableWindow::ShowMessages()
{
  if (!messageWindow()->isHidden())
    messageWindow()->hide();  
  else  
    messageWindow()->show();  
  showMessagesAction->setOn(!messageWindow()->isHidden());
}

void CTableWindow::init(bool ok)
{
  if (ok)
  {    
    connect(tableInsertAction, SIGNAL(activated()), this, SLOT(insertRow()));
    connect(tableDeleteAction, SIGNAL(activated()), this, SLOT(deleteRow()));
    connect(tablePrimaryKeyAction, SIGNAL(activated()), this, SLOT(setPk()));
    connect(Fields, SIGNAL(contextMenuRequested(int, int, const QPoint &)), this, SLOT(showContextMenu(int, int, const QPoint &)));
    connect(Fields, SIGNAL(clicked(int, int, int, const QPoint &)), this, SLOT(fieldsClicked(int, int, int, const QPoint &)));
    connect(Fields, SIGNAL(valueChanged(int, int)), this, SLOT(fieldsValueChanged(int, int)));    
    connect(fileSaveAction, SIGNAL(activated()), this, SLOT(save()));
    connect(fileSaveAsAction, SIGNAL(activated()), this, SLOT(saveAs()));
    connect(fileReloadAction, SIGNAL(activated()), this, SLOT(reset()));
    connect(editCopyAction, SIGNAL(activated()), this, SLOT(copy()));
    connect(editPasteAction, SIGNAL(activated()), this, SLOT(paste()));
    connect(Tab, SIGNAL(currentChanged(QWidget *)), indexesTab, SLOT(refresh()));
    connect(indexesTab, SIGNAL(primaryKey(bool, int)), this, SLOT(addPrimaryIcon(bool, int)));
    connect(this, SIGNAL(valueChanged()), indexesTab, SLOT(refresh()));
    connect(this, SIGNAL(valueChanged()), alterTableOptionsTab, SLOT(refresh()));
    reset();
  }
  else
  {
    setWindowCaption();
    tableInsertAction->setEnabled(false);
    tableDeleteAction->setEnabled(false);
    tablePrimaryKeyAction->setEnabled(false);
    fileSaveAction->setEnabled(false);
    fileSaveAsAction->setEnabled(false);
    fileReloadAction->setEnabled(false);
    editCopyAction->setEnabled(false);
    Tab->setTabEnabled(indexesTab, false);
    Tab->setTabEnabled(tablePropertiesTab, false);
    Tab->setTabEnabled(alterTableOptionsTab, false);
    Tab->setTabEnabled(fieldPropertiesTab, false);
  }
  connect(showMessagesAction, SIGNAL(activated()), this, SLOT(ShowMessages()));
  connect(messageWindow(), SIGNAL(visibilityChanged(bool)), showMessagesAction, SLOT(setOn(bool)));
  connect(fileCloseAction, SIGNAL(activated()), this, SLOT(close()));  
}
